import 'dart:convert';
import 'dart:io';

import 'package:at_commons/at_commons.dart';
import 'package:at_utils/at_logger.dart';
import 'package:at_client/at_client.dart';

class AuthKeyType {
  static const String pkamPublicKey = 'aesPkamPublicKey';
  static const String pkamPrivateKey = 'aesPkamPrivateKey';
  static const String encryptionPublicKey = 'aesEncryptPublicKey';
  static const String encryptionPrivateKey = 'aesEncryptPrivateKey';
  static const String selfEncryptionKey = 'selfEncryptionKey';
}

Future<void> _saveKeysToKeyStore(var _atClient, Map _atKeysMap) async {
  await _atClient
      ?.getLocalSecondary()
      ?.putValue(AT_PKAM_PUBLIC_KEY, _atKeysMap[AuthKeyType.pkamPublicKey]);

  await _atClient?.getLocalSecondary()?.putValue(
      AT_PKAM_PRIVATE_KEY, _atKeysMap[AuthKeyType.pkamPrivateKey]);

  await _atClient?.getLocalSecondary()?.putValue(
      AT_ENCRYPTION_PUBLIC_KEY,
      _atKeysMap[AuthKeyType.encryptionPublicKey]);

  await _atClient?.getLocalSecondary()?.putValue(
      AT_ENCRYPTION_PRIVATE_KEY,
      _atKeysMap[AuthKeyType.encryptionPrivateKey]);

  await _atClient?.getLocalSecondary()?.putValue(
      AT_ENCRYPTION_SELF_KEY, _atKeysMap[AuthKeyType.selfEncryptionKey]);
}

Map _loadKeysFromFile(String atKeysFilePath) {
  File atKeysFile = File(atKeysFilePath);
  String atAuthData = atKeysFile.readAsStringSync();
  Map encryptedKeys = jsonDecode(atAuthData);

  var decryptionKey = encryptedKeys[AuthKeyType.selfEncryptionKey];
  Map _atKeysMap = {
    AuthKeyType.pkamPublicKey: EncryptionUtil.decryptValue(
        encryptedKeys[AuthKeyType.pkamPublicKey], decryptionKey),
    AuthKeyType.pkamPrivateKey: EncryptionUtil.decryptValue(
        encryptedKeys[AuthKeyType.pkamPrivateKey], decryptionKey),
    AuthKeyType.encryptionPublicKey: EncryptionUtil.decryptValue(
        encryptedKeys[AuthKeyType.encryptionPublicKey], decryptionKey),
    AuthKeyType.encryptionPrivateKey: EncryptionUtil.decryptValue(
        encryptedKeys[AuthKeyType.encryptionPrivateKey], decryptionKey),
    AuthKeyType.selfEncryptionKey: decryptionKey,
  };
  return _atKeysMap;
}

main() async {
  var atSign = '@murali🛠';
  AtSignLogger.root_level = 'finest';

  AtSignLogger logger = AtSignLogger('issue_709_main');

  Map keys = _loadKeysFromFile('keys/${atSign}_key.atKeys');

  var atPref = AtClientPreference()
    ..hiveStoragePath = 'storage/$atSign/hive/client'
    ..commitLogPath = 'storage/$atSign/hive/client/commit'
    ..isLocalStoreRequired = true
    ..privateKey = keys[AuthKeyType.pkamPrivateKey]
    ..namespace='issue709'
    ..rootDomain = 'vip.ve.atsign.zone';

  var atMgr = AtClientManager.getInstance();
  await atMgr.setCurrentAtSign(atSign, atPref.namespace, atPref);
  var atClient = atMgr.atClient;

  await _saveKeysToKeyStore(atClient, keys);

  atMgr.syncService.sync();

  await Future.delayed(Duration(milliseconds: 30000));
  int mockFrequency = 1000;
  while (true) {
    var metadata = Metadata()
      ..isPublic = true..isEncrypted = false..namespaceAware=true
    // 10 mins
      ..ttl=600000;
    var atKey = AtKey()
      ..key = 'public.mock77'
      ..sharedBy = atSign
      ..namespace='issue709'
      ..metadata = metadata;

    logger.info('Putting value $mockFrequency to $atKey');
    await atClient.put(atKey, mockFrequency.toString());

    var atValue = await atClient.get(atKey);
    logger.info("Getting the key $atValue");

    await Future.delayed(Duration(milliseconds: 10000));

    mockFrequency += 1;
  }
}
